from django.contrib import admin
from django.utils import timezone
from django.utils.translation import ugettext_lazy as _

from . import filters
from .. import models


@admin.register(models.RefreshToken)
class RefreshTokenAdmin(admin.ModelAdmin):
    list_display = ['user', 'token', 'created', 'revoked', 'is_expired']
    list_filter = (filters.RevokedFilter, filters.ExpiredFilter)
    raw_id_fields = ('user',)
    search_fields = ('token',)
    actions = ('revoke',)

    def revoke(self, request, queryset):
        queryset.update(revoked=timezone.now())

    revoke.short_description = _('Revoke selected %(verbose_name_plural)s')

    def is_expired(self, obj):
        return obj.is_expired()

    is_expired.boolean = True
    is_expired.short_description = _('is expired')



from django.contrib.admin import SimpleListFilter
from django.utils.translation import ugettext_lazy as _

__all__ = [
    'ExpiredFilter',
    'RevokedFilter',
]


class BooleanListFilter(SimpleListFilter):

    def lookups(self, request, model_admin):
        return (
            ('yes', _('Yes')),
            ('no', _('No')),
        )


class ExpiredFilter(BooleanListFilter):
    title = _('Expired')
    parameter_name = 'expired'

    def queryset(self, request, queryset):
        if self.value() == 'yes':
            return queryset.expired().filter(expired=True)

        if self.value() == 'no':
            return queryset.expired().filter(expired=False)


class RevokedFilter(BooleanListFilter):
    title = _('Revoked')
    parameter_name = 'revoked'

    def queryset(self, request, queryset):
        if self.value() == 'yes':
            return queryset.filter(revoked__isnull=False)

        if self.value() == 'no':
            return queryset.filter(revoked__isnull=True)



import binascii
import os
from calendar import timegm

from django.conf import settings
from django.db import models
from django.utils import timezone
from django.utils.translation import ugettext_lazy as _

from . import managers, signals
from ..settings import jwt_settings
from .shortcuts import create_refresh_token


class AbstractRefreshToken(models.Model):
    id = models.BigAutoField(primary_key=True)

    user = models.ForeignKey(
        settings.AUTH_USER_MODEL,
        on_delete=models.CASCADE,
        related_name='refresh_token',
        verbose_name=_('user'))

    token = models.CharField(_('token'), max_length=255, editable=False)

    created = models.DateTimeField(_('created'), auto_now_add=True)
    revoked = models.DateTimeField(_('revoked'), null=True, blank=True)

    objects = managers.RefreshTokenQuerySet.as_manager()

    class Meta:
        abstract = True
        verbose_name = _('Refresh token')
        verbose_name_plural = _('Refresh tokens')
        unique_together = ('token', 'revoked')

    def __str__(self):
        return self.token

    def save(self, *args, **kwargs):
        if not self.token:
            self.token = self.generate_token()
        return super(AbstractRefreshToken, self).save(*args, **kwargs)

    def generate_token(self):
        return binascii.hexlify(
            os.urandom(jwt_settings.JWT_REFRESH_TOKEN_N_BYTES),
        ).decode()

    def is_expired(self, context=None):
        orig_iat = timegm(self.created.timetuple())
        return jwt_settings.JWT_REFRESH_EXPIRED_HANDLER(orig_iat, context)

    def revoke(self):
        self.revoked = timezone.now()
        self.save(update_fields=['revoked'])

        signals.refresh_token_revoked.send(
            sender=AbstractRefreshToken,
            refresh_token=self)

    def rotate(self):
        refresh_token = create_refresh_token(user=self.user)

        signals.refresh_token_rotated.send(
            sender=AbstractRefreshToken,
            refresh_token=self)

        return refresh_token


class RefreshToken(AbstractRefreshToken):
    """RefreshToken default model"""



from django.http import JsonResponse
from django.contrib.auth import authenticate
from django.utils.cache import patch_vary_headers

from .exceptions import JSONWebTokenError
from .utils import get_authorization_header


class JSONWebTokenMiddleware(object):

    def __init__(self, get_response):
        self.get_response = get_response

    def __call__(self, request):
        if get_authorization_header(request) is not None:
            if not hasattr(request, 'user') or request.user.is_anonymous:
                try:
                    user = authenticate(request=request)
                except JSONWebTokenError as err:
                    return JsonResponse({
                        'errors': [{'message': str(err)}]
                    }, status=401)

                if user is not None:
                    request.user = request._cached_user = user

        response = self.get_response(request)
        patch_vary_headers(response, ('Authorization',))
        return response



from functools import wraps

import six
from django.contrib.auth import authenticate, get_user_model
from django.utils.translation import ugettext as _
from graphql import GraphQLResolveInfo

from promise import Promise, is_thenable

from . import exceptions
from .shortcuts import get_token
from .settings import jwt_settings
from .utils import get_authorization_header
from .refresh_token.shortcuts import create_refresh_token

__all__ = [
    'user_passes_test',
    'login_required',
    'staff_member_required',
    'permission_required',
    'token_auth',
]


def context(f):
    def decorator(func):
        def wrapper(*args, **kwargs):
            info = next(arg for arg in args
                        if isinstance(arg, GraphQLResolveInfo))
            return func(info.context, *args, **kwargs)

        return wrapper

    return decorator


def user_passes_test(test_func):
    def decorator(f):
        @wraps(f)
        @context(f)
        def wrapper(context, *args, **kwargs):
            if test_func(context.get('request').user):
                return f(*args, **kwargs)
            raise exceptions.PermissionDenied()

        return wrapper

    return decorator


login_required = user_passes_test(lambda u: u.is_authenticated)
staff_member_required = user_passes_test(lambda u: u.is_active and u.is_staff)


def permission_required(perm):
    def check_perms(user):
        if isinstance(perm, six.string_types):
            perms = (perm,)
        else:
            perms = perm

        if user.has_perms(perms):
            return True
        return False

    return user_passes_test(check_perms)


def token_auth(f):
    @wraps(f)
    def wrapper(root, info, password, **kwargs):
        def on_resolve(values):
            user, payload = values
            payload['token'] = get_token(user, info.context)
            if jwt_settings.JWT_LONG_RUNNING_REFRESH_TOKEN:
                payload['refresh_token'] = create_refresh_token(user).token
            return payload

        username = kwargs.get(get_user_model().USERNAME_FIELD)

        if get_authorization_header(info.context.get('request')) is not None:
            del info.context.get('request').META[jwt_settings.JWT_AUTH_HEADER]

        user = authenticate(info.context.get('request'),
                            username=username,
                            password=password)

        if user is None:
            raise exceptions.JSONWebTokenError(
                _('Please, enter valid credentials'))

        if hasattr(info.context.get('request'), 'user'):
            info.context.get('request').user = user

        result = f(root, info, **kwargs)
        values = (user, result)
        # Improved mutation with thenable check
        if is_thenable(result):
            return Promise.resolve(values).then(on_resolve)
        return on_resolve(values)

    return wrapper



from calendar import timegm

from django.utils.translation import ugettext as _

from .. import exceptions
from ..settings import jwt_settings
from .shortcuts import get_refresh_token


def resolve_refresh_token(obj, info, refresh_token, **kwargs):
    context = info.context
    refresh_token = get_refresh_token(refresh_token)

    if refresh_token.is_expired(context):
        raise exceptions.JSONWebTokenError(_('Refresh token is expired'))

    payload = jwt_settings.JWT_PAYLOAD_HANDLER(refresh_token.user, context)
    token = jwt_settings.JWT_ENCODE_HANDLER(payload, context)
    refreshed_token = refresh_token.rotate().token

    return {'token': token, 'payload': payload, 'refresh_token': refreshed_token}


def resolve_revoke(obj, info, refresh_token, **kwargs):
    refresh_token = get_refresh_token(refresh_token)
    refresh_token.revoke()

    return {'revoked': timegm(refresh_token.revoked.timetuple())}



from django.apps import AppConfig
from django.utils.translation import ugettext_lazy as _


class RefreshTokenConfig(AppConfig):
    name = 'ariadne_jwt.refresh_token'
    verbose_name = _('Refresh token')



from django.contrib.auth import authenticate
from django.core.handlers.wsgi import WSGIRequest
from django.test import Client, RequestFactory, testcases

import ariadne
from graphql import ExecutionResult

from .settings import jwt_settings
from .shortcuts import get_token


class SchemaRequestFactory(RequestFactory):

    def __init__(self, **defaults):
        super(SchemaRequestFactory, self).__init__(**defaults)
        self._schema = None

    def schema(self, type_defs, *resolvers, directives=None):
        self._schema = ariadne.make_executable_schema(type_defs, *resolvers, directives=directives)

    def execute(self, context, query, variables):
        response = ariadne.graphql_sync(self._schema, {'query': query, 'variables': variables}, context_value=context)
        response = ExecutionResult(response[1].get('data'), response[1].get('errors'))
        return response


class JSONWebTokenClient(SchemaRequestFactory, Client):

    def __init__(self, **defaults):
        super(JSONWebTokenClient, self).__init__(**defaults)
        self._credentials = {}

    def request(self, **request):
        request = WSGIRequest(self._base_environ(**request))
        request.user = authenticate(request)
        return request

    def credentials(self, **kwargs):
        self._credentials = kwargs

    def execute(self, query, variables=None, **extra):
        if variables is None:
            variables = {}
        extra.update(self._credentials)
        variables.update(extra)
        context = {'request': self.post('/', **extra)}
        return super(JSONWebTokenClient, self).execute(context, query, variables)

    def authenticate(self, user):
        self._credentials = {
            jwt_settings.JWT_AUTH_HEADER: '{0} {1}'.format(
                jwt_settings.JWT_AUTH_HEADER_PREFIX,
                get_token(user)),
        }

    def logout(self):
        self._credentials.pop(jwt_settings.JWT_AUTH_HEADER, None)


class JSONWebTokenTestCase(testcases.TestCase):
    client_class = JSONWebTokenClient



from .mutations import (resolve_verify, resolve_refresh, resolve_revoke, resolve_token_auth, jwt_schema)
from .scalar import GenericScalar

__all__ = ['resolve_verify', 'resolve_refresh', 'resolve_revoke', 'resolve_token_auth', 'jwt_schema', 'GenericScalar', ]



from django.utils.translation import ugettext as _

from . import exceptions
from .settings import jwt_settings
from .decorators import token_auth
from .utils import get_payload, get_user_by_payload
from .refresh_token.mutations import resolve_refresh_token, resolve_revoke

__all__ = ['resolve_verify', 'resolve_refresh', 'resolve_revoke', 'resolve_token_auth', 'jwt_schema']

jwt_schema = '''
    scalar GenericScalar
    
    type VerifyToken {
        payload: GenericScalar
    }
    type RefreshToken {
        token: String
        refresh_token: String
        payload: GenericScalar
    }
    type TokenAuth {
        token: String
        refresh_token: String
        payload: GenericScalar
    }
    type RevokeToken {
        revoked: Int
    }
'''


def resolve_verify(obj, info, token, **kwargs):
    return {'payload': get_payload(token, info.context)}


def resolve_keep_alive_refresh_token(obj, info, token, **kwargs):
    context = info.context
    payload = get_payload(token, context)
    user = get_user_by_payload(payload)
    orig_iat = payload.get('origIat')

    if not orig_iat:
        raise exceptions.JSONWebTokenError(_('origIat field is required'))

    if jwt_settings.JWT_REFRESH_EXPIRED_HANDLER(orig_iat, context):
        raise exceptions.JSONWebTokenError(_('Refresh has expired'))

    payload = jwt_settings.JWT_PAYLOAD_HANDLER(user, context)
    payload['origIat'] = orig_iat
    token = jwt_settings.JWT_ENCODE_HANDLER(payload, context)

    return {'token': token, 'payload': payload}


def resolve_refresh(obj, info, token, **kwargs):
    return (resolve_refresh_token(obj, info, token, **kwargs)
            if jwt_settings.JWT_LONG_RUNNING_REFRESH_TOKEN
            else resolve_keep_alive_refresh_token(obj, info, token, **kwargs))


@token_auth
def resolve_token_auth(obj, info, **kwargs):
    return {}



from django.core.management.base import BaseCommand
from django.db.models import Q
from django.template.defaultfilters import pluralize

from ariadne_jwt.refresh_token.shortcuts import get_refresh_token_model


class Command(BaseCommand):
    help = 'Clears refresh tokens'

    def add_arguments(self, parser):
        parser.add_argument(
            '--expired',
            action='store_true',
            help='Clears expired tokens',
        )

    def handle(self, expired, *args, **options):
        qs = get_refresh_token_model().objects
        query = Q(revoked__isnull=False)

        if expired:
            qs = qs.expired()
            query |= Q(expired=True)

        deleted, _ = qs.filter(query).delete()

        msg = 'Successfully deleted {} token{}'.format(
            deleted,
            pluralize(deleted))

        self.stdout.write(self.style.SUCCESS(msg))



from ariadne.scalars import ScalarType
from graphql import (StringValueNode, BooleanValueNode, IntValueNode,
                     FloatValueNode, ListValueNode, ObjectValueNode)

try:
    from graphql.type.scalars import (MAX_INT as GRAPHQL_MAX_INT,
                                      MIN_INT as GRAPHQL_MIN_INT)
except ImportError:
    from graphql.type.scalars import GRAPHQL_MAX_INT, GRAPHQL_MIN_INT

GenericScalar = ScalarType('GenericScalar')


@GenericScalar.serializer
@GenericScalar.value_parser
def identity(value):
    return value


@GenericScalar.literal_parser
def parse_literal(ast):
    if isinstance(ast, (StringValueNode, BooleanValueNode)):
        return ast.value
    elif isinstance(ast, IntValueNode):
        num = int(ast.value)
        if GRAPHQL_MAX_INT <= num <= GRAPHQL_MIN_INT:
            return num
    elif isinstance(ast, FloatValueNode):
        return float(ast.value)
    elif isinstance(ast, ListValueNode):
        return [parse_literal(value) for value in ast.values]
    elif isinstance(ast, ObjectValueNode):
        return {
            field.name.value: parse_literal(field.value)
            for field in ast.fields
        }
    else:
        return None



from django.db import models
from django.db.models import Case
from django.db.models import Value as V
from django.db.models import When
from django.utils import timezone

from ..settings import jwt_settings


class RefreshTokenQuerySet(models.QuerySet):

    def expired(self):
        expires = timezone.now() - jwt_settings.JWT_REFRESH_EXPIRATION_DELTA
        return self.annotate(
            expired=Case(
                When(created__lt=expires, then=V(True)),
                output_field=models.BooleanField(),
                default=V(False),
            ),
        )



from .shortcuts import get_user_by_token
from .utils import get_authorization_header, get_user_by_natural_key


class JSONWebTokenBackend(object):

    def authenticate(self, request=None, **credentials):
        if request is None:
            return None

        token = get_authorization_header(request)
        if token is not None:
            return get_user_by_token(token, context={'request': request})
        return None

    def get_user(self, user_id):
        return get_user_by_natural_key(user_id)



from .settings import jwt_settings
from .utils import get_payload, get_user_by_payload
from .refresh_token.shortcuts import create_refresh_token, get_refresh_token

__all__ = [
    'get_token',
    'get_user_by_token',
    'get_refresh_token',
    'create_refresh_token',
]


def get_token(user, context=None, **extra):
    payload = jwt_settings.JWT_PAYLOAD_HANDLER(user, context)
    payload.update(extra)
    return jwt_settings.JWT_ENCODE_HANDLER(payload, context)


def get_user_by_token(token, context=None):
    payload = get_payload(token, context)
    return get_user_by_payload(payload)



from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):
    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='RefreshToken',
            fields=[
                ('id', models.BigAutoField(primary_key=True, serialize=False)),
                ('token', models.CharField(editable=False, max_length=255, verbose_name='token')),
                ('created', models.DateTimeField(auto_now_add=True, verbose_name='created')),
                ('revoked', models.DateTimeField(blank=True, null=True, verbose_name='revoked')),
                ('user', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='refresh_token',
                                           to=settings.AUTH_USER_MODEL, verbose_name='user')),
            ],
            options={
                'verbose_name': 'Refresh token',
                'verbose_name_plural': 'Refresh tokens',
                'abstract': False,
            },
        ),
        migrations.AlterUniqueTogether(
            name='refreshtoken',
            unique_together={('token', 'revoked')},
        ),
    ]



from django.apps import apps
from django.utils.translation import ugettext as _

from ..exceptions import JSONWebTokenError
from ..settings import jwt_settings


def get_refresh_token_model():
    return apps.get_model(jwt_settings.JWT_REFRESH_TOKEN_MODEL)


def get_refresh_token(token):
    RefreshToken = get_refresh_token_model()

    try:
        return RefreshToken.objects.get(token=token, revoked__isnull=True)
    except RefreshToken.DoesNotExist:
        raise JSONWebTokenError(_('Invalid refresh token'))


def create_refresh_token(user):
    return get_refresh_token_model().objects.create(user=user)
